#-*- coding: utf-8 -*-
# 1行目は日本語コードの指定
#
# algeq.py
# 代数方程式の解（ニュートン法）
# 2022.7

from math import *
from cmath import *
from random import *
from sys import *

# 多項式は昇順の係数リストで表現する。
# 例えば [3,0,2,1] は x^3 + 2 x^2 + 3 を表すこととする。
# ゼロ多項式は [] で表す。

# 次数
def deg(f):
    return (len(f) - 1)

# 多項式の微分を返す
def polder(f):
    if deg(f) < 0:
        g = []
    else:
        n = len(f)
        g = [i * f[i] for i in range(1, n)]
    return g

# 多項式 f(x) の x = a での値を返す
def poleval(f, a):
    w = 0
    d = deg(f)
    for i in range(d, -1, -1):
        w = a * w + f[i]
    return w

# 多項式 f(x) が一次因子 x - a をもつときに商 f(x) / (x - a) を返す
def poldivlinfac(f, a):
    d = deg(f)
    g = []
    k = 0
    for i in range(d, 0, -1):
        k = f[i] + a * k
        g = [k] + g
    return g

# Newton 法を用いて f(x) = 0 の解をひとつ求める
def polnewton(f):
    seed()          # 乱数の初期化
    e = pow(10, -12)    # 許容誤差
    g = polder(f)
    ok = 0
    while not ok:
        a = 0
        b = complex(random(), random())  # 初期値はランダムな複素数
        m = 0
        while m < 1024:          # 最大 1024 回まで反復
            a = b
            b = a - poleval(f, a) / poleval(g, a)
            m += 1
            if abs(a - b) <= e:       # 収束していればOK
                ok = 1
                break
            
    return b

# Newton 法を繰り返し用いて f(x) = 0 の解を全て求める
def polroots(f):
    r = []
    while deg(f) > 0:
        a = polnewton(f)
        f = poldivlinfac(f, a)
        r.append(a)
        if 0:
            print('[', end = ' ')
            complexpolwrite([-a, 1])
            print('] * [', end = ' ')
            complexpolwrite(f)
            print(']')
    return r

# 実係数多項式を出力する
def polwrite(f):
    d = deg(f)
    if d < 0:
        print('0')
    else:
        i = d
        while i >= 0:
            c = f[i]
            if c != 0:
                if c > 0:
                    if i < d:
                        print('+', end = ' ')
                else:
                    print('-', end = ' ')
                c = abs(c)
                if c != 1 or i == 0:
                    print(c, end = ' ')
                if i > 1:
                    print('x^%d' % i, end = ' ')
                if i == 1:
                    print('x', end=' ')
            i -= 1
    return

# 複素係数多項式を出力する
def complexpolwrite(f):
    d = deg(f)
    if d < 0:
        print('0')
    else:
        i = d
        while i >= 0:
            c = f[i]
            if c != 0:
                if c != 0:
                    if i < d:
                        print('+', end = ' ')
                if c != 1 or i == 0:
                    print(c, end = ' ')
                if i > 1:
                    print('x^%d' % i, end = ' ')
                if i == 1:
                    print('x', end = ' ')
            i -= 1
    return

# 多項式を入力させる
def polinput():
    d = eval(input('次数 = '))
    f = []
    i = d
    while i >= 0:
        print(i, end = ' ')
        c = eval(input('次の係数 = '))
        f = [c] + f
        i -= 1
    return f

# 複素数の実部
def re(z):
    return z.real

# 複素数の虚部
def im(z):
    return z.imag

# メインルーチン
print('代数方程式 f(x) = 0 の解を求めます。')
print('f(x) を入力してください。')
f = polinput()
print
print('f(x) =' , end = ' ')
polwrite(f)
print
print
roots = polroots(f)
print('f(x) = 0 の解')
for r in roots:
    print(r)

